#include "StdAfx.h"
#include ".\simpletextscriptletplugin.h"
#include "ScriptletConstant.h"
#include "SimpleTextShape.h"
#include "StateTextShape.h"
#include "XMLTextShape.h"
#include "XMLUtil.h"

REGISTER_SCRIPTLETCOMPONENTPLUGIN_CREATOR(SCRIPTLET_SIMPLETEXT_NODE, CSimpleTextScriptletPlugin);

CSimpleTextScriptletPlugin::CSimpleTextScriptletPlugin(void)
{
}

CSimpleTextScriptletPlugin::~CSimpleTextScriptletPlugin(void)
{
}
BOOL CSimpleTextScriptletPlugin::ModifyComponent(CContainer* pParent, CComponent* pTarget, TiXmlElement* pNode)
{
	BOOL bSuccess = __super::ModifyComponent(pParent, pTarget, pNode);
	if (bSuccess)
	{
		CSimpleTextShape* pSimpleText = reinterpret_cast<CSimpleTextShape*>(pTarget);
		if (pSimpleText == NULL)
		{
			return FALSE;
		}
		TCHAR szValue[SCRIPTLET_MAX_VALUE] = {0};
		long lValue = 0;

		//text
		if (XMLUtil::GetAttrValue(pNode, CT2CA(SCRIPTLET_TEXT_NODE), szValue, SCRIPTLET_MAX_VALUE) ||
			XMLUtil::GetValueAsAttr(pNode, CT2CA(SCRIPTLET_TEXT_NODE), szValue, SCRIPTLET_MAX_VALUE))
		{
			LPCTSTR szText = LoadString(szValue);
			pSimpleText->SetText(szText);
		}
		//text.halign
		if (XMLUtil::GetAttrValue(pNode, CT2CA(SCRIPTLET_TEXT_HALIGN_NODE), szValue, SCRIPTLET_MAX_VALUE) ||
			XMLUtil::GetValueAsAttr(pNode, CT2CA(SCRIPTLET_TEXT_HALIGN_NODE), szValue, SCRIPTLET_MAX_VALUE))
		{
			int nHAlign = PFA_LEFT;
			if (_tcsicmp(szValue, SCRIPTLET_CENTER_VALUE) == 0)
			{
				nHAlign = PFA_CENTER;
			}
			else if (_tcsicmp(szValue, SCRIPTLET_RIGHT_VALUE) == 0)
			{
				nHAlign = PFA_RIGHT;
			}
			pSimpleText->SetHAlign(nHAlign);
		}
		//text.valign
		if (XMLUtil::GetAttrValue(pNode, CT2CA(SCRIPTLET_TEXT_VALIGN_NODE), szValue, SCRIPTLET_MAX_VALUE) ||
			XMLUtil::GetValueAsAttr(pNode, CT2CA(SCRIPTLET_TEXT_VALIGN_NODE), szValue, SCRIPTLET_MAX_VALUE))
		{
			int nVAlign = PFA_TOP;
			if (_tcsicmp(szValue, SCRIPTLET_CENTER_VALUE) == 0)
			{
				nVAlign = PFA_CENTER;
			}
			else if (_tcsicmp(szValue, SCRIPTLET_BOTTOM_VALUE) == 0)
			{
				nVAlign = PFA_BOTTOM;
			}
			pSimpleText->SetVAlign(nVAlign);
		}
		
		//trimming
		if (XMLUtil::GetAttrValue(pNode, CT2CA(L"trimming"), szValue, SCRIPTLET_MAX_VALUE) ||
			XMLUtil::GetValueAsAttr(pNode, CT2CA(L"trimming"), szValue, SCRIPTLET_MAX_VALUE))
		{
			BOOL bTrimming = (_tcscmp(szValue, L"true") == 0);
			pSimpleText->SetTrimming(bTrimming);
		}

		//font.foreground
		if (XMLUtil::GetAttrValue(pNode, CT2CA(L"font.foreground"), szValue, SCRIPTLET_MAX_VALUE) ||
			XMLUtil::GetValueAsAttr(pNode, CT2CA(L"font.foreground"), szValue, SCRIPTLET_MAX_VALUE))
		{
			COLORREF clrFore = XMLUtil::ColorFromString(szValue);
			pSimpleText->SetFontColor(clrFore);
		}

		//font
		CStringA strFontElement = pNode->Value();
		strFontElement.Append(".");
		strFontElement.Append(CT2CA(SCRIPTLET_FONT_NODE));
		TiXmlElement* pFontNode = pNode->FirstChildElement(strFontElement);
		if (pFontNode)
		{
			//fontname
			if (XMLUtil::GetAttrValue(pFontNode, CT2CA(SCRIPTLET_FONTNAME_NODE), szValue, SCRIPTLET_MAX_VALUE) ||
				XMLUtil::GetValueAsSubAttr(pFontNode, CT2CA(SCRIPTLET_FONTNAME_NODE), szValue, SCRIPTLET_MAX_VALUE))
			{
				LPCTSTR szFontName = LoadString(szValue);
				pSimpleText->SetFontName(szFontName, HANGEUL_CHARSET);
			}
			//fontisze
			if (XMLUtil::GetAttrValue(pFontNode, CT2CA(SCRIPTLET_FONTSIZE_NODE), szValue, SCRIPTLET_MAX_VALUE) ||
				XMLUtil::GetValueAsSubAttr(pFontNode, CT2CA(SCRIPTLET_FONTSIZE_NODE), szValue, SCRIPTLET_MAX_VALUE))
			{
				int nFontSize = _ttoi(szValue);
				pSimpleText->SetFontSize(nFontSize);
			}
			//bold
			if (XMLUtil::GetAttrValue(pFontNode, CT2CA(SCRIPTLET_BOLD_NODE), szValue, SCRIPTLET_MAX_VALUE) ||
				XMLUtil::GetValueAsSubAttr(pFontNode, CT2CA(SCRIPTLET_BOLD_NODE), szValue, SCRIPTLET_MAX_VALUE))
			{
				BOOL bBold = _tcsicmp(szValue, SCRIPTLET_TRUE_VALUE) == 0;
				pSimpleText->SetBold(bBold);
			}
			//italic
			if (XMLUtil::GetAttrValue(pFontNode, CT2CA(SCRIPTLET_ITALIC_NODE), szValue, SCRIPTLET_MAX_VALUE) ||
				XMLUtil::GetValueAsSubAttr(pFontNode, CT2CA(SCRIPTLET_ITALIC_NODE), szValue, SCRIPTLET_MAX_VALUE))
			{
				BOOL bItalic = _tcsicmp(szValue, SCRIPTLET_TRUE_VALUE) == 0;
				pSimpleText->SetItalic(bItalic);
			}
			//underline
			if (XMLUtil::GetAttrValue(pFontNode, CT2CA(SCRIPTLET_UNDERLINE_NODE), szValue, SCRIPTLET_MAX_VALUE) ||
				XMLUtil::GetValueAsSubAttr(pFontNode, CT2CA(SCRIPTLET_UNDERLINE_NODE), szValue, SCRIPTLET_MAX_VALUE))
			{
				BOOL bItalic = _tcsicmp(szValue, SCRIPTLET_TRUE_VALUE) == 0;
				pSimpleText->SetUnderline(bItalic);
			}
			//foreground
			if (XMLUtil::GetAttrValue(pFontNode, CT2CA(SCRIPTLET_FOREGROUND_NODE), szValue, SCRIPTLET_MAX_VALUE) ||
				XMLUtil::GetValueAsSubAttr(pFontNode, CT2CA(SCRIPTLET_FOREGROUND_NODE), szValue, SCRIPTLET_MAX_VALUE))
			{
				COLORREF clrFore = XMLUtil::ColorFromString(szValue);
				pSimpleText->SetFontColor(clrFore);
			}
		}
	}
	return bSuccess;
}
CComponent* CSimpleTextScriptletPlugin::_CreateComponent(TiXmlElement* pNode)
{
	CComponent* pComponent = new CSimpleTextShape;
	return pComponent;
}

REGISTER_SCRIPTLETCOMPONENTPLUGIN_CREATOR(SCRIPTLET_XMLTEXT_NODE, CXMLTextScriptletPlugin);
BOOL CXMLTextScriptletPlugin::ModifyComponent(CContainer* pParent, CComponent* pTarget, TiXmlElement* pNode)
{
	BOOL bSuccess = __super::ModifyComponent(pParent, pTarget, pNode);
	if (bSuccess)
	{
		CXMLTextShape* pXMLText = reinterpret_cast<CXMLTextShape*>(pTarget);
		if (pXMLText == NULL)
		{
			return FALSE;
		}
		TCHAR szValue[SCRIPTLET_MAX_VALUE] = {0};
		long lValue = 0;

		//highlight
		if (XMLUtil::GetAttrValue(pNode, CT2CA(SCRIPTLET_HIGHLIGHT_NODE), szValue, SCRIPTLET_MAX_VALUE) ||
			XMLUtil::GetValueAsAttr(pNode, CT2CA(SCRIPTLET_HIGHLIGHT_NODE), szValue, SCRIPTLET_MAX_VALUE))
		{
			COLORREF clrHighlight = XMLUtil::ColorFromString(szValue);
			pXMLText->SetHighlightFontColor(clrHighlight);
		}
	}
	return bSuccess;
}
CComponent* CXMLTextScriptletPlugin::_CreateComponent(TiXmlElement* pNode)
{
	CComponent* pComponent = new CXMLTextShape;
	return pComponent;
}

REGISTER_SCRIPTLETCOMPONENTPLUGIN_CREATOR(SCRIPTLET_STATETEXT_NODE, CStateTextScriptletPlugin);
BOOL CStateTextScriptletPlugin::ModifyComponent(CContainer* pParent, CComponent* pTarget, TiXmlElement* pNode)
{
	BOOL bSuccess = __super::ModifyComponent(pParent, pTarget, pNode);
	if (bSuccess)
	{
		StateTextShape* pStateText = reinterpret_cast<StateTextShape*>(pTarget);
		if (pStateText == NULL)
		{
			return FALSE;
		}
		TCHAR szValue[SCRIPTLET_MAX_VALUE] = {0};
		long lValue = 0;

		//overcolor
		if (XMLUtil::GetAttrValue(pNode, CT2CA(SCRIPTLET_OVERCOLOR_NODE), szValue, SCRIPTLET_MAX_VALUE) ||
			XMLUtil::GetValueAsAttr(pNode, CT2CA(SCRIPTLET_OVERCOLOR_NODE), szValue, SCRIPTLET_MAX_VALUE))
		{
			COLORREF clrOver = XMLUtil::ColorFromString(szValue);
			pStateText->SetOverTextColor(clrOver);
		}
		//presscolor
		if (XMLUtil::GetAttrValue(pNode, CT2CA(SCRIPTLET_PRESSCOLOR_NODE), szValue, SCRIPTLET_MAX_VALUE) ||
			XMLUtil::GetValueAsAttr(pNode, CT2CA(SCRIPTLET_PRESSCOLOR_NODE), szValue, SCRIPTLET_MAX_VALUE))
		{
			COLORREF clrPress = XMLUtil::ColorFromString(szValue);
			pStateText->SetPressTextColor(clrPress);
		}
		//disablecolor
		if (XMLUtil::GetAttrValue(pNode, CT2CA(SCRIPTLET_DISABLECOLOR_NODE), szValue, SCRIPTLET_MAX_VALUE) ||
			XMLUtil::GetValueAsAttr(pNode, CT2CA(SCRIPTLET_DISABLECOLOR_NODE), szValue, SCRIPTLET_MAX_VALUE))
		{
			COLORREF clrDisable = XMLUtil::ColorFromString(szValue);
			pStateText->SetDiableTextColor(clrDisable);
		}
	}
	return bSuccess;
}
CComponent* CStateTextScriptletPlugin::_CreateComponent(TiXmlElement* pNode)
{
	CComponent* pComponent = new StateTextShape;
	return pComponent;
}